ID = "id"
URL = "url"


ERROR = "ERROR"
INITIALIZING = "INITIALIZING"
IN_PROGRESS = "IN_PROGRESS"

AUTHORIZATION = "Authorization"
BEARER_TOKEN = "Bearer %s"
D_REALM = "d-realm"
ACCESS_TOKEN = "access_token"
REFRESH_TOKEN = "refresh_token"

TRAIN_ITERATIONS = "trainIterations"
STEPS = "steps"
ORIGIN_DATASOURCE_ID = "originDataSourceId"
TARGET_DATASOURCE_ID = "targetDataSourceId"
MODELS = "models"


NEW_PRIVATE_ID = -999
NEW_SAMPLE_ID = -888

SPARK_WRITE_OPTIONS_OVERWRITE = "overwrite"
SPARK_WRITE_OPTIONS_APPEND = "append"

ENHANCE_ML_TARGET_COLUMN = "targetColumnName"
ENHANCE_ML_TARGET_TABLE = "targetTableName"
